import { baseDecode, baseEncode } from 'borsh';
declare class Call {
    origin_public_key: Uint8Array;
    receipt_public_key: Uint8Array;
    encrypted_egress: boolean;
    transaction_action: number;
    amount: number | null;
    contract_name: Uint8Array | null;
    method: Uint8Array | null;
    args: Uint8Array | null;
    constructor(config: {
        origin_public_key: Uint8Array;
        receipt_public_key: Uint8Array;
        encrypted_egress: boolean;
        transaction_action: number;
        amount: number | null;
        contract_name: Uint8Array | null;
        method: Uint8Array | null;
        args: Uint8Array | null;
    });
}
declare class Calls {
    ops: Call[];
    shard_id: number;
    block_number: number | null;
    constructor(config: {
        ops: Call[];
        shard_id: number;
        block_number: number | null;
    });
}
declare class Outcome {
    view_result_log: Uint8Array[];
    view_result: Uint8Array | null;
    view_error: Uint8Array | null;
    outcome_logs: Uint8Array[];
    outcome_receipt_ids: Uint8Array[];
    outcome_token_burnt: number;
    outcome_executor_id: Uint8Array;
    outcome_status: Uint8Array | null;
    constructor(config: {
        view_result_log: Uint8Array[];
        view_result: Uint8Array | null;
        view_error: Uint8Array | null;
        outcome_logs: Uint8Array[];
        outcome_receipt_ids: Uint8Array[];
        outcome_token_burnt: number;
        outcome_executor_id: Uint8Array;
        outcome_status: Uint8Array | null;
    });
}
declare class Outcomes {
    ops: Outcome[];
    state_root: Uint8Array;
    constructor(config: {
        ops: Outcome[];
        state_root: Uint8Array;
    });
}
declare class RawOutcomes {
    ops: Outcome[];
    state_root: Uint8Array;
    state_patch: Uint8Array;
    constructor(config: {
        ops: Outcome[];
        state_root: Uint8Array;
        state_patch: Uint8Array;
    });
}
declare class LocalMetadata {
    shard_id: number[];
    high_local_block: number;
    latest_state_root: Uint8Array;
    constructor(config: {
        shard_id: number[];
        high_local_block: number;
        latest_state_root: Uint8Array;
    });
}
declare class Block {
    shard_id: number;
    block_number: number;
    calls: number[];
    contracts: string[];
    constructor(config: {
        shard_id: number;
        block_number: number;
        calls: number[];
        contracts: string[];
    });
}
declare class Shard {
    high_remote_synced_block_index: number;
    high_remote_confirmed_block_index: number;
    constructor(config: {
        high_remote_synced_block_index: number;
        high_remote_confirmed_block_index: number;
    });
}
declare class ShardMetadata {
    shard_key: Uint8Array;
    shard_members: string[];
    beacon_index: number;
    threshold: number;
    constructor(config: {
        shard_key: Uint8Array;
        shard_members: string[];
        beacon_index: number;
        threshold: number;
    });
}
declare class Contract {
    home_shard: number;
    wasm_blob: string;
    deployment_call: Calls;
    deployment_call_index: number;
    constructor(config: {
        home_shard: number;
        wasm_blob: string;
        deployment_call: Calls;
        deployment_call_index: number;
    });
}
declare class ExecutionSummary {
    high_local_execution_block: number;
    constructor(config: {
        high_local_execution_block: number;
    });
}
declare class BlockSummary {
    block_number: number;
    block_state_root: Uint8Array;
    contract_state_patch_from_previous_block: Uint8Array;
    call_state_patch_from_previous_block: Uint8Array;
    constructor(config: {
        block_number: number;
        block_state_root: Uint8Array;
        contract_state_patch_from_previous_block: Uint8Array;
        call_state_patch_from_previous_block: Uint8Array;
    });
}
declare const blockSummarySchema: Map<typeof BlockSummary, {
    kind: string;
    fields: (string | (string | number)[])[][];
}>;
declare const blockSchema: Map<typeof Block, {
    kind: string;
    fields: (string | {
        kind: string;
        type: string[];
    })[][];
}>;
declare const callSchema: Map<typeof Call, {
    kind: string;
    fields: ((string | (string | number)[])[] | (string | {
        kind: string;
        type: string;
    })[] | (string | {
        kind: string;
        type: string[];
    })[])[];
}>;
declare const callsSchema: Map<any, any>;
declare const outcomeSchema: Map<typeof Outcome, {
    kind: string;
    fields: ((string | string[])[] | (string | {
        kind: string;
        type: string[][];
    })[] | (string | {
        kind: string;
        type: string[];
    })[] | (string | (string | number)[][])[])[];
}>;
declare const outcomesSchema: Map<any, any>;
declare const rawOutcomesSchema: Map<any, any>;
declare const contractSchema: Map<any, any>;
declare const shardMetadataSchema: Map<typeof ShardMetadata, {
    kind: string;
    fields: (string | (string | number)[])[][];
}>;
declare const shardSchema: Map<typeof Shard, {
    kind: string;
    fields: string[][];
}>;
declare const localMetadataSchema: Map<typeof LocalMetadata, {
    kind: string;
    fields: (string | (string | number)[])[][];
}>;
declare const executionSummarySchema: Map<typeof ExecutionSummary, {
    kind: string;
    fields: string[][];
}>;
declare const buildCall: (call: Call) => string;
declare const buildBlock: (block: Block) => string;
declare const buildContract: (contract: Contract) => string;
declare const buildCalls: (calls: Calls) => string;
declare const buildOutcome: (outcome: Outcome) => string;
declare const buildOutcomes: (outcomes: Outcomes) => string;
declare const buildShard: (shard: Shard) => string;
declare const buildShardMetadata: (shardMetadata: ShardMetadata) => string;
declare const buildLocalMetadata: (a: LocalMetadata) => string;
declare const buildExecutionSummary: (a: ExecutionSummary) => string;
declare const buildBlockSummary: (a: BlockSummary) => string;
declare const buildRawOutcomes: (a: RawOutcomes) => string;
declare const parseCall: (buf: string) => Call;
declare const parseCalls: (buf: string) => Calls;
declare const parseOutcome: (buf: string) => Outcome;
declare const parseOutcomes: (buf: string) => Outcomes;
declare const parseBlock: (buf: string) => Block;
declare const parseContract: (buf: string) => Contract;
declare const parseShard: (buf: string) => Shard;
declare const parseShardMetadata: (buf: string) => ShardMetadata;
declare const parseLocalMetadata: (buf: string) => LocalMetadata;
declare const parseExecutionSummary: (buf: string) => ExecutionSummary;
declare const parseBlockSummary: (buf: string) => BlockSummary;
declare const parseRawOutcomes: (buf: string) => RawOutcomes;
export { Call, callSchema, buildCall, parseCall, Calls, callsSchema, buildCalls, parseCalls, Outcome, outcomeSchema, buildOutcome, parseOutcome, Outcomes, outcomesSchema, buildOutcomes, parseOutcomes, Block, blockSchema, buildBlock, parseBlock, Contract, contractSchema, buildContract, parseContract, Shard, shardSchema, buildShard, parseShard, ShardMetadata, shardMetadataSchema, buildShardMetadata, parseShardMetadata, LocalMetadata, localMetadataSchema, buildLocalMetadata, parseLocalMetadata, ExecutionSummary, executionSummarySchema, buildExecutionSummary, parseExecutionSummary, BlockSummary, blockSummarySchema, buildBlockSummary, parseBlockSummary, RawOutcomes, rawOutcomesSchema, buildRawOutcomes, parseRawOutcomes, baseEncode, baseDecode };
