// Copyright 2021-2022 @skyekiwi/s-contract authors & contributors
// SPDX-License-Identifier: Apache-2.0
import { baseDecode, baseEncode, deserialize, serialize } from 'borsh';
/* eslint-disable sort-keys, camelcase */

class Call {
  // public transaction_action: 'create_account' | 'call' | 'transfer' | 'view_method_call' | 'deploy'
  constructor(config) {
    this.origin_public_key = void 0;
    this.receipt_public_key = void 0;
    this.encrypted_egress = void 0;
    this.transaction_action = void 0;
    this.amount = void 0;
    this.contract_name = void 0;
    this.method = void 0;
    this.args = void 0;
    this.origin_public_key = config.origin_public_key;
    this.receipt_public_key = config.receipt_public_key;
    this.encrypted_egress = config.encrypted_egress;
    this.transaction_action = config.transaction_action;
    this.amount = config.amount;
    this.contract_name = config.contract_name;
    this.method = config.method;
    this.args = config.args;
  }

}

class Calls {
  constructor(config) {
    this.ops = void 0;
    this.shard_id = void 0;
    this.block_number = void 0;
    this.ops = config.ops;
    this.shard_id = config.shard_id;
    this.block_number = config.block_number;
  }

}

class Outcome {
  constructor(config) {
    this.view_result_log = void 0;
    this.view_result = void 0;
    this.view_error = void 0;
    this.outcome_logs = void 0;
    this.outcome_receipt_ids = void 0;
    this.outcome_token_burnt = void 0;
    this.outcome_executor_id = void 0;
    this.outcome_status = void 0;
    this.view_result_log = config.view_result_log;
    this.view_result = config.view_result;
    this.view_error = config.view_error;
    this.outcome_logs = config.outcome_logs;
    this.outcome_receipt_ids = config.outcome_receipt_ids;
    this.outcome_token_burnt = config.outcome_token_burnt;
    this.outcome_executor_id = config.outcome_executor_id;
    this.outcome_status = config.outcome_status;
  }

}

class Outcomes {
  constructor(config) {
    this.ops = void 0;
    this.state_root = void 0;
    this.ops = config.ops;
    this.state_root = config.state_root;
  }

}

class RawOutcomes {
  constructor(config) {
    this.ops = void 0;
    this.state_root = void 0;
    this.state_patch = void 0;
    this.ops = config.ops;
    this.state_root = config.state_root;
    this.state_patch = config.state_patch;
  }

}

class LocalMetadata {
  constructor(config) {
    this.shard_id = void 0;
    this.high_local_block = void 0;
    this.latest_state_root = void 0;
    this.shard_id = config.shard_id;
    this.high_local_block = config.high_local_block;
    this.latest_state_root = config.latest_state_root;
  }

}

class Block {
  constructor(config) {
    this.shard_id = void 0;
    this.block_number = void 0;
    this.calls = void 0;
    this.contracts = void 0;
    this.shard_id = config.shard_id;
    this.block_number = config.block_number;
    this.calls = config.calls;
    this.contracts = config.contracts;
  }

}

class Shard {
  constructor(config) {
    this.high_remote_synced_block_index = void 0;
    this.high_remote_confirmed_block_index = void 0;
    this.high_remote_synced_block_index = config.high_remote_synced_block_index;
    this.high_remote_confirmed_block_index = config.high_remote_confirmed_block_index;
  }

}

class ShardMetadata {
  constructor(config) {
    this.shard_key = void 0;
    this.shard_members = void 0;
    this.beacon_index = void 0;
    this.threshold = void 0;
    this.shard_key = config.shard_key;
    this.shard_members = config.shard_members;
    this.beacon_index = config.beacon_index;
    this.threshold = config.threshold;
  }

}

class Contract {
  constructor(config) {
    this.home_shard = void 0;
    this.wasm_blob = void 0;
    this.deployment_call = void 0;
    this.deployment_call_index = void 0;
    this.home_shard = config.home_shard;
    this.wasm_blob = config.wasm_blob;
    this.deployment_call = config.deployment_call;
    this.deployment_call_index = config.deployment_call_index;
  }

}

class ExecutionSummary {
  constructor(config) {
    this.high_local_execution_block = void 0;
    this.high_local_execution_block = config.high_local_execution_block;
  }

}

class BlockSummary {
  constructor(config) {
    this.block_number = void 0;
    this.block_state_root = void 0;
    this.contract_state_patch_from_previous_block = void 0;
    this.call_state_patch_from_previous_block = void 0;
    this.block_number = config.block_number;
    this.block_state_root = config.block_state_root;
    this.contract_state_patch_from_previous_block = config.contract_state_patch_from_previous_block;
    this.call_state_patch_from_previous_block = config.call_state_patch_from_previous_block;
  }

}

const blockSummarySchema = new Map([[BlockSummary, {
  kind: 'struct',
  fields: [['block_number', 'u32'], ['block_state_root', ['u8', 32]], ['contract_state_patch_from_previous_block', ['u8']], ['call_state_patch_from_previous_block', ['u8']]]
}]]);
const blockSchema = new Map([[Block, {
  kind: 'struct',
  fields: [['shard_id', 'u32'], ['block_number', 'u32'], ['calls', {
    kind: 'option',
    type: ['u32']
  }], ['contracts', {
    kind: 'option',
    type: ['string']
  }]]
}]]);
const callSchema = new Map([[Call, {
  kind: 'struct',
  fields: [['origin_public_key', ['u8', 32]], ['receipt_public_key', ['u8', 32]], ['encrypted_egress', 'u8'], ['transaction_action', 'u8'], ['amount', {
    kind: 'option',
    type: 'u32'
  }], ['contract_name', {
    kind: 'option',
    type: ['u8']
  }], ['method', {
    kind: 'option',
    type: ['u8']
  }], ['args', {
    kind: 'option',
    type: ['u8']
  }]]
}]]);
const callsSchema = new Map();
callsSchema.set(Calls, {
  kind: 'struct',
  fields: [['ops', [Call]], ['shard_id', 'u32'], ['block_number', {
    kind: 'option',
    type: 'u32'
  }]]
});
callsSchema.set(Call, callSchema.get(Call));
const outcomeSchema = new Map([[Outcome, {
  kind: 'struct',
  fields: [['view_result_log', {
    kind: 'option',
    type: [['u8']]
  }], ['view_result', {
    kind: 'option',
    type: ['u8']
  }], ['view_error', {
    kind: 'option',
    type: ['u8']
  }], ['outcome_logs', [['u8']]], ['outcome_receipt_ids', [['u8', 32]]], ['outcome_token_burnt', 'u32'], ['outcome_executor_id', ['u8']], ['outcome_status', {
    kind: 'option',
    type: ['u8']
  }]]
}]]);
const outcomesSchema = new Map();
outcomesSchema.set(Outcomes, {
  kind: 'struct',
  fields: [['ops', [Outcome]], ['state_root', ['u8', 32]]]
});
outcomesSchema.set(Outcome, outcomeSchema.get(Outcome));
const rawOutcomesSchema = new Map();
rawOutcomesSchema.set(RawOutcomes, {
  kind: 'struct',
  fields: [['ops', [Outcome]], ['state_root', ['u8', 32]], ['state_patch', ['u8']]]
});
rawOutcomesSchema.set(Outcome, outcomeSchema.get(Outcome));
const contractSchema = new Map();
contractSchema.set(Contract, {
  kind: 'struct',
  fields: [['home_shard', 'u32'], ['wasm_blob', 'string'], ['deployment_call', Calls], ['deployment_call_index', 'u32']]
});
contractSchema.set(Calls, callsSchema.get(Calls));
contractSchema.set(Call, callSchema.get(Call));
const shardMetadataSchema = new Map([[ShardMetadata, {
  kind: 'struct',
  fields: [['shard_key', ['u8', 32]], ['shard_members', ['string']], ['beacon_index', 'u32'], ['threshold', 'u32']]
}]]);
const shardSchema = new Map([[Shard, {
  kind: 'struct',
  fields: [['high_remote_synced_block_index', 'u32'], ['high_remote_confirmed_block_index', 'u32']]
}]]);
const localMetadataSchema = new Map([[LocalMetadata, {
  kind: 'struct',
  fields: [['shard_id', ['u32']], ['high_local_block', 'u32'], ['latest_state_root', ['u8', 32]]]
}]]);
const executionSummarySchema = new Map([[ExecutionSummary, {
  kind: 'struct',
  fields: [['high_local_execution_block', 'u32']]
}]]); // ser

const buildCall = call => {
  const buf = serialize(callSchema, call);
  return baseEncode(buf);
};

const buildBlock = block => {
  const buf = serialize(blockSchema, block);
  return baseEncode(buf);
};

const buildContract = contract => {
  const buf = serialize(contractSchema, contract);
  return baseEncode(buf);
};

const buildCalls = calls => {
  const buf = serialize(callsSchema, calls);
  return baseEncode(buf);
};

const buildOutcome = outcome => {
  const buf = serialize(outcomeSchema, outcome);
  return baseEncode(buf);
};

const buildOutcomes = outcomes => {
  const buf = serialize(outcomesSchema, outcomes);
  return baseEncode(buf);
};

const buildShard = shard => {
  const buf = serialize(shardSchema, shard);
  return baseEncode(buf);
};

const buildShardMetadata = shardMetadata => {
  const buf = serialize(shardMetadataSchema, shardMetadata);
  return baseEncode(buf);
};

const buildLocalMetadata = a => {
  const buf = serialize(localMetadataSchema, a);
  return baseEncode(buf);
};

const buildExecutionSummary = a => {
  const buf = serialize(executionSummarySchema, a);
  return baseEncode(buf);
};

const buildBlockSummary = a => {
  const buf = serialize(blockSummarySchema, a);
  return baseEncode(buf);
};

const buildRawOutcomes = a => {
  const buf = serialize(rawOutcomesSchema, a);
  return baseEncode(buf);
}; // de


const parseCall = buf => {
  const c = deserialize(callSchema, Call, baseDecode(buf));
  c.encrypted_egress = !!c.encrypted_egress;
  return c;
};

const parseCalls = buf => {
  const cs = deserialize(callsSchema, Calls, baseDecode(buf));

  for (let i = 0; i < cs.ops.length; i++) {
    cs.ops[i].encrypted_egress = !!cs.ops[i].encrypted_egress;
  }

  return cs;
};

const parseOutcome = buf => {
  return deserialize(outcomeSchema, Outcome, baseDecode(buf));
};

const parseOutcomes = buf => {
  return deserialize(outcomesSchema, Outcomes, baseDecode(buf));
};

const parseBlock = buf => {
  return deserialize(blockSchema, Block, baseDecode(buf));
};

const parseContract = buf => {
  return deserialize(contractSchema, Contract, baseDecode(buf));
};

const parseShard = buf => {
  return deserialize(shardSchema, Shard, baseDecode(buf));
};

const parseShardMetadata = buf => {
  return deserialize(shardMetadataSchema, ShardMetadata, baseDecode(buf));
};

const parseLocalMetadata = buf => {
  return deserialize(localMetadataSchema, LocalMetadata, baseDecode(buf));
};

const parseExecutionSummary = buf => {
  return deserialize(executionSummarySchema, ExecutionSummary, baseDecode(buf));
};

const parseBlockSummary = buf => {
  return deserialize(blockSummarySchema, BlockSummary, baseDecode(buf));
};

const parseRawOutcomes = buf => {
  return deserialize(rawOutcomesSchema, RawOutcomes, baseDecode(buf));
};

export { Call, callSchema, buildCall, parseCall, Calls, callsSchema, buildCalls, parseCalls, Outcome, outcomeSchema, buildOutcome, parseOutcome, Outcomes, outcomesSchema, buildOutcomes, parseOutcomes, Block, blockSchema, buildBlock, parseBlock, Contract, contractSchema, buildContract, parseContract, Shard, shardSchema, buildShard, parseShard, ShardMetadata, shardMetadataSchema, buildShardMetadata, parseShardMetadata, LocalMetadata, localMetadataSchema, buildLocalMetadata, parseLocalMetadata, ExecutionSummary, executionSummarySchema, buildExecutionSummary, parseExecutionSummary, BlockSummary, blockSummarySchema, buildBlockSummary, parseBlockSummary, RawOutcomes, rawOutcomesSchema, buildRawOutcomes, parseRawOutcomes, baseEncode, baseDecode };